using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;


namespace WiiMultipointGrid
{
    class Particle
    {
        public Vector3 position = new Vector3();
        public Vector3 lastPosition = new Vector3();
        public Vector3 lastPosition2 = new Vector3();
        public Vector3 velocity = new Vector3();
        public Vector3 acceleration = new Vector3();
        public float age = 0;
        public float drag = 0;//% of velocity lost per second
        public float maxVelocity = float.PositiveInfinity;
        public float maxAcceleration = float.PositiveInfinity;

        public Particle() { }

        public float getVelocityMagnitude(){
            return (float)Math.Sqrt(velocity.X * velocity.X + velocity.Y * velocity.Y);
        }
        public float getAccelerationMagnitude()
        {
            return (float)Math.Sqrt(acceleration.X * acceleration.X + acceleration.Y * acceleration.Y);
        }

        public void enforceMaxVelocity()
        {
            float mag = getVelocityMagnitude();
            if (mag > maxVelocity)
            {
                float s = maxVelocity / mag;
                velocity.X *= s;
                velocity.Y *= s;
            }
        }
        public void enforceMaxAcceleration()
        {
            float mag = getAccelerationMagnitude();
            if (mag > maxVelocity)
            {
                float s = maxAcceleration / mag;
                velocity.X *= s;
                velocity.Y *= s;
            }
        }

        public void gravitate(Particle p, float g)
        {
            if (p == null)
            {
                acceleration.X = 0;
                acceleration.Y = 0;
                acceleration.Z = 0;
                return;
            }

            double dist = distance(p);
            double dg = dist * dist * dist;
            float dx = p.position.X - position.X;
            float dy = p.position.Y - position.Y;
            float dz = p.position.Z - position.Z;
            acceleration.X = (float)(g * dx / dg);
            acceleration.Y = (float)(g * dy / dg);
            acceleration.Z = (float)(g * dy / dg);
        }

        public double distance(Particle p)
        {
            float dx = p.position.X - position.X;
            float dy = p.position.Y - position.Y;
            float dz = p.position.Z - position.Z;
            return Math.Sqrt(dx * dx + dy * dy +dz * dz);
        }

        public void simulate(float dtime)
        {
            lastPosition2.X = lastPosition.X;
            lastPosition2.Y = lastPosition.Y;
            lastPosition.X = position.X;
            lastPosition.Y = position.Y;
            position.X += velocity.X * dtime + .5f * acceleration.X * dtime * dtime;
            position.Y += velocity.Y * dtime + .5f * acceleration.Y * dtime * dtime;
            velocity.X += acceleration.X * dtime;
            velocity.Y += acceleration.Y * dtime;

            //since we are simple simulator, just do drag when we aren't accelerating
                //drag is based on velocity magnitude??
                float d = (1.0f - drag * dtime);
                //            float mag = getVelocityMagnitude();
                if (d < 0)
                    d = 0;
                velocity.X *= d;
                velocity.Y *= d;
            age += dtime;
        }
    }
}
