using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace WiiMultipointGrid
{
    class ParticleFountain
    {
        public Particle source;
        Particle[] particles;
        VertexBuffer buffer = null;
        float maxAge = 3;
        public float defaultVelocity = .4f;
        Random rand = new Random();

        int color;

        public ParticleFountain(Device dev, Particle source, int particleCount, float maxAge, float drag, int color)
        {
            this.source = source;
            this.maxAge = maxAge;
            this.color = color;
            particles = new Particle[particleCount];

            buffer = new VertexBuffer(typeof(CustomVertex.PositionColored), 4 * particleCount, dev, 0, CustomVertex.PositionColored.Format, Pool.Managed);
            CustomVertex.PositionColored[] verts;
            verts = (CustomVertex.PositionColored[])buffer.Lock(0, 0); // Lock the buffer (which will return our structs)

            for (int i = 0; i < particleCount; i++)
            {
                particles[i] = new Particle();
                regenerateParticle(particles[i]);
                particles[i].age = maxAge*(float)i / particleCount;
                particles[i].drag = drag;

                verts[i*4].Position = particles[i].lastPosition;
                verts[i*4].Color = color;
                verts[i * 4+1].Position = particles[i].position;
                verts[i * 4+1].Color = color;
                verts[i * 4+2].Position = particles[i].lastPosition;
                verts[i * 4+2].Color = color;
                verts[i * 4 + 3].Position = particles[i].lastPosition2;
                verts[i * 4 + 3].Color = color;
            }
            buffer.Unlock();
        }

        public void regenerateParticle(Particle p)
        {
            p.age = 0;
            
            p.position.X = source.position.X;
            p.position.Y = source.position.Y;
            p.lastPosition.X = source.position.X;
            p.lastPosition.Y = source.position.Y;
            p.lastPosition2.X = source.position.X;
            p.lastPosition2.Y = source.position.Y;

            p.velocity.X = defaultVelocity*(rand.Next(10000) / 10000.0f +.5f);
            p.velocity.Y = defaultVelocity * (rand.Next(10000) / 10000.0f + .5f);
        }
        public void gravitate(Particle sink,float g)
        {

            for (int i = 0; i < particles.Length; i++)
            {
                if (sink != null)
                {
                    if (particles[i].distance(sink) < .05)
                        regenerateParticle(particles[i]);
                }

                particles[i].gravitate(sink, g);
            }
        }

        public void update(float dtime)
        {
            CustomVertex.PositionColored[] verts;
            verts = (CustomVertex.PositionColored[])buffer.Lock(0, 0); // Lock the buffer (which will return our structs)
            for (int i = 0; i < particles.Length; i++)
            {
                particles[i].simulate(dtime);
                if (particles[i].age > maxAge)
                {
                    regenerateParticle(particles[i]);
                }
                verts[i * 4].Position = particles[i].lastPosition;
                verts[i * 4].Color = color;
                verts[i * 4 + 1].Position = particles[i].position;
                verts[i * 4 + 1].Color = color;
                verts[i * 4 + 2].Position = particles[i].lastPosition;
                verts[i * 4 + 2].Color = color;
                verts[i * 4 + 3].Position = particles[i].lastPosition2;
                verts[i * 4 + 3].Color = color;
            }
            buffer.Unlock();
        }
        public void Render(Device device)
        {
            device.Transform.World = Matrix.Identity;
            device.SetStreamSource(0, buffer, 0);
            device.VertexFormat = CustomVertex.PositionColored.Format;
            device.DrawPrimitives(PrimitiveType.LineList, 0, particles.Length);
        }
    }
}
